@@ -87,6 +87,7 @@ enum {
 enum {
 	ITEMS_AVAILABLE,
 	FINISHED,
+	RAISE_ERROR,
 	LAST_SIGNAL
 };
 
@@ -243,20 +244,6 @@ retry_synchronously (TrackerDecorator *decorator,
 	}
 }
 
-static void
-tag_success (TrackerDecorator *decorator,
-             GPtrArray        *commit_buffer)
-{
-	guint i;
-
-	for (i = 0; i < commit_buffer->len; i++) {
-		TrackerExtractInfo *info;
-
-		info = g_ptr_array_index (commit_buffer, i);
-		tracker_error_report_delete (tracker_extract_info_get_file (info));
-	}
-}
-
 static void
 decorator_commit_cb (GObject      *object,
                      GAsyncResult *result,
@@ -265,6 +252,7 @@ decorator_commit_cb (GObject      *object,
 	TrackerDecoratorPrivate *priv;
 	TrackerDecorator *decorator;
 	TrackerBatch *batch;
+	g_autoptr (GError) error = NULL;
 
 	decorator = user_data;
 	priv = tracker_decorator_get_instance_private (decorator);
@@ -272,11 +260,12 @@ decorator_commit_cb (GObject      *object,
 
 	priv->updating = FALSE;
 
-	if (!tracker_batch_execute_finish (batch, result, NULL)) {
+	if (!tracker_batch_execute_finish (batch, result, &error)) {
+		if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+			return;
+
 		g_debug ("SPARQL error detected in batch, retrying one by one");
 		retry_synchronously (decorator, priv->commit_buffer);
-	} else {
-		tag_success (decorator, priv->commit_buffer);
 	}
 
 	g_clear_pointer (&priv->commit_buffer, g_ptr_array_unref);
@@ -321,6 +310,7 @@ decorator_commit_info (TrackerDecorator *decorator)
 	                             decorator_commit_cb,
 	                             decorator);
 	decorator_update_state (decorator, NULL, TRUE);
+	g_object_unref (batch);
 
 	return TRUE;
 }
@@ -518,6 +508,7 @@ decorator_item_cache_remove (TrackerDecorator *decorator,
 
 		g_queue_remove (&priv->item_cache, info);
 		tracker_decorator_info_unref (info);
+		break;
 	}
 }
 
@@ -531,6 +522,7 @@ decorator_cache_items_cb (GObject      *object,
 	g_autoptr (TrackerSparqlCursor) cursor = NULL;
 	TrackerDecoratorInfo *info;
 	g_autoptr (GError) error = NULL;
+	gboolean queue_was_empty;
 
 	cursor = tracker_sparql_statement_execute_finish (TRACKER_SPARQL_STATEMENT (object),
 	                                                  result, &error);
@@ -539,6 +531,8 @@ decorator_cache_items_cb (GObject      *object,
 
 	decorator_commit_info (decorator);
 
+	queue_was_empty = g_queue_is_empty (&priv->item_cache);
+
 	if (error) {
 		g_warning ("Could not get unextracted files: %s", error->message);
 		return;
@@ -553,44 +547,45 @@ decorator_cache_items_cb (GObject      *object,
 		decorator_start (decorator);
 	} else if (g_queue_is_empty (&priv->item_cache) && priv->processing) {
 		decorator_finish (decorator);
+	} else if (queue_was_empty) {
+		g_signal_emit (decorator, signals[ITEMS_AVAILABLE], 0);
 	}
 }
 
 static void
-bind_graphs (TrackerDecorator *decorator,
-             int              *arg_counter,
-             gboolean          priority)
+bind_graph_limits (TrackerDecorator *decorator,
+                   gboolean          priority)
 {
 	TrackerDecoratorPrivate *priv;
-	TrackerSparqlConnection *conn;
-	TrackerNamespaceManager *namespaces;
-	const gchar *graphs[] = {
-		"tracker:Audio",
-		"tracker:Pictures",
-		"tracker:Video",
-		"tracker:Software",
-		"tracker:Documents",
+	const gchar *graphs[][3] = {
+		{ "tracker:Audio", "audioHigh", "audioLow" },
+		{ "tracker:Pictures", "picturesHigh", "picturesLow" },
+		{ "tracker:Video", "videoHigh", "videoLow" },
+		{ "tracker:Software", "softwareHigh", "softwareLow" },
+		{ "tracker:Documents", "documentsHigh", "documentsLow" },
 	};
 	guint i;
 
 	priv = tracker_decorator_get_instance_private (decorator);
-	conn = tracker_miner_get_connection (TRACKER_MINER (decorator));
-	namespaces = tracker_sparql_connection_get_namespace_manager (conn);
 
 	for (i = 0; i < G_N_ELEMENTS (graphs); i++) {
+		const gchar *graph = graphs[i][0];
+		const gchar *high_limit = graphs[i][1];
+		const gchar *low_limit = graphs[i][2];
 		gboolean is_priority;
-		g_autofree gchar *arg = NULL, *expanded = NULL;
 
 		is_priority = priv->priority_graphs &&
-			g_strv_contains ((const gchar * const *) priv->priority_graphs, graphs[i]);
+			g_strv_contains ((const gchar * const *) priv->priority_graphs, graph);
 
-		if (priority != is_priority)
-			continue;
-
-		arg = g_strdup_printf ("graph%d", (*arg_counter) + 1);
-		expanded = tracker_namespace_manager_expand_uri (namespaces, graphs[i]);
-		tracker_sparql_statement_bind_string (priv->remaining_items_query, arg, expanded);
-		(* arg_counter)++;
+		/* Graphs with high priority get unbound high limit and 0 low limit,
+		 * graphs with regular priority get the opposite.
+		 */
+		tracker_sparql_statement_bind_int (priv->remaining_items_query,
+		                                   high_limit,
+		                                   is_priority ? -1 : 0);
+		tracker_sparql_statement_bind_int (priv->remaining_items_query,
+		                                   low_limit,
+		                                   is_priority ? 0 : -1);
 	}
 }
 
@@ -598,7 +593,7 @@ static void
 decorator_query_next_items (TrackerDecorator *decorator)
 {
 	TrackerDecoratorPrivate *priv;
-	gint offset = 0, n_graph = 0;
+	gint offset = 0;
 
 	priv = tracker_decorator_get_instance_private (decorator);
 
@@ -610,10 +605,7 @@ decorator_query_next_items (TrackerDecorator *decorator)
 	if (!priv->remaining_items_query)
 		priv->remaining_items_query = load_statement (decorator, "get-items.rq");
 
-	/* Bind all graphs, in priority order */
-	bind_graphs (decorator, &n_graph, TRUE);
-	bind_graphs (decorator, &n_graph, FALSE);
-
+	bind_graph_limits (decorator, TRUE);
 	tracker_sparql_statement_bind_int (priv->remaining_items_query,
 	                                   "offset", offset);
 	tracker_sparql_statement_bind_int (priv->remaining_items_query,
@@ -887,6 +879,16 @@ tracker_decorator_class_init (TrackerDecoratorClass *klass)
 		              G_STRUCT_OFFSET (TrackerDecoratorClass, finished),
 		              NULL, NULL, NULL,
 		              G_TYPE_NONE, 0);
+
+	signals[RAISE_ERROR] =
+		g_signal_new ("raise-error",
+		              G_OBJECT_CLASS_TYPE (object_class),
+		              G_SIGNAL_RUN_LAST,
+		              0, NULL, NULL, NULL,
+		              G_TYPE_NONE, 3,
+			      G_TYPE_FILE,
+			      G_TYPE_STRING,
+			      G_TYPE_STRING);
 }
 
 static void
