@@ -23,7 +23,8 @@
 #include <gio/gio.h>
 
 #include "tracker-extract.h"
-#include "tracker-decorator-fs.h"
+#include "tracker-decorator.h"
+#include "tracker-extract-persistence.h"
 
 G_BEGIN_DECLS
 
@@ -38,19 +39,20 @@ typedef struct TrackerExtractDecorator TrackerExtractDecorator;
 typedef struct TrackerExtractDecoratorClass TrackerExtractDecoratorClass;
 
 struct TrackerExtractDecorator {
-	TrackerDecoratorFS parent_instance;
+	TrackerDecorator parent_instance;
 };
 
 struct TrackerExtractDecoratorClass {
-        TrackerDecoratorFSClass parent_class;
+	TrackerDecoratorClass parent_class;
 };
 
 GType              tracker_extract_decorator_get_type (void) G_GNUC_CONST;
 
-TrackerDecorator * tracker_extract_decorator_new (TrackerSparqlConnection  *connection,
-                                                  TrackerExtract           *extractor,
-                                                  GCancellable             *cancellable,
-                                                  GError                  **error);
+TrackerDecorator * tracker_extract_decorator_new (TrackerSparqlConnection   *connection,
+                                                  TrackerExtract            *extractor,
+                                                  TrackerExtractPersistence *persistence,
+                                                  GCancellable              *cancellable,
+                                                  GError                   **error);
 
 G_END_DECLS
 
